/*
 * @(#)CollapsiblePaneGroup.java 9/13/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.pane.CollapsiblePane;

import javax.swing.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;

public class CollapsiblePaneGroup implements Serializable, PropertyChangeListener {

    // the list of _panes participating in this group
    protected Vector _panes = new Vector();

    /**
     * The current selection.
     */
    CollapsiblePane _selection = null;

    /**
     * Creates a new <code>ButtonGroup</code>.
     */
    public CollapsiblePaneGroup() {
    }

    /**
     * Adds the button to the group.
     *
     * @param b the button to be added
     */
    public void add(CollapsiblePane b) {
        if (b == null) {
            return;
        }
        _panes.addElement(b);
        b.addPropertyChangeListener(this);

        if (b.isEmphasized()) {
            if (_selection == null) {
                _selection = b;
            }
            else {
                b.setEmphasized(false);
            }
        }
    }

    /**
     * Removes the button from the group.
     *
     * @param b the button to be removed
     */
    public void remove(AbstractButton b) {
        if (b == null) {
            return;
        }
        _panes.removeElement(b);
        if (b.getModel() == _selection) {
            _selection = null;
        }
        b.getModel().setGroup(null);
    }

    /**
     * Returns all the _panes that are participating in
     * this group.
     *
     * @return an <code>Enumeration</code> of the _panes in this group
     */
    public Enumeration getElements() {
        return _panes.elements();
    }

    /**
     * Returns the model of the selected button.
     *
     * @return the selected button model
     */
    public CollapsiblePane getSelection() {
        return _selection;
    }

    /**
     * Sets the selected value for the <code>ButtonModel</code>.
     * Only one button in the group may be selected at a time.
     *
     * @param m the <code>ButtonModel</code>
     * @param b <code>true</code> if this button is to be
     *          selected, otherwise <code>false</code>
     */
    public void setSelected(CollapsiblePane m, boolean b) {
        if (b && m != null && m != _selection) {
            CollapsiblePane oldSelection = _selection;
            _selection = m;
            if (oldSelection != null) {
                oldSelection.setEmphasized(false);
            }
            m.setEmphasized(true);
        }
    }

    /**
     * Returns whether a <code>ButtonModel</code> is selected.
     *
     * @return <code>true</code> if the button is selected,
     *         otherwise returns <code>false</code>
     */
    public boolean isSelected(ButtonModel m) {
        return (m == _selection);
    }

    /**
     * Returns the number of _panes in the group.
     *
     * @return the button count
     * @since 1.3
     */
    public int getButtonCount() {
        if (_panes == null) {
            return 0;
        }
        else {
            return _panes.size();
        }
    }

    public void propertyChange(PropertyChangeEvent evt) {
        if (CollapsiblePane.EMPHASIZED_PROPERTY.equals(evt.getPropertyName()) && Boolean.TRUE.equals(evt.getNewValue())) {
            Enumeration e = getElements();
            while (e.hasMoreElements()) {
                Object o = e.nextElement();
                if (o != evt.getSource() && o instanceof CollapsiblePane) {
                    ((CollapsiblePane) o).setEmphasized(false);
                }
            }
        }
    }
}
