/*
 * @(#)ColorChooserDemo.java 2/12/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.combobox.ColorChooserPanel;
import com.jidesoft.combobox.ColorComboBox;
import com.jidesoft.converter.ObjectConverterManager;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JideBoxLayout;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 * Demoed Component: {@link ColorComboBox}, {@link ColorChooserPanel}
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: Jide L&F extension required
 */
public class ColorChooserDemo extends AbstractDemo {
    public ColorComboBox _colorComboBox;
    protected JLabel _messageLabel;

    public ColorChooserDemo() {
    }

    public String getName() {
        return "ColorChooser and ColorComboBox Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_GRIDS;
    }

    public Component getOptionsPanel() {
        return createOptionsPanel();
    }

    public String getDescription() {
        return "This is a demo of ColorChooserPanel and ColorComboBox. Please choose different options in the options pane and click on the drop down button of ColorComboBox to see the difference.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.combobox.ColorComboBox\n" +
                "com.jidesoft.combobox.ColorChooserPanel";
    }

    public Component getDemoPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new JideBoxLayout(panel, BoxLayout.Y_AXIS));

        _colorComboBox = createColorComboBox();
        _messageLabel = new JLabel();

        panel.add(_colorComboBox);
        panel.add(Box.createVerticalStrut(6));
        panel.add(_messageLabel);
        panel.add(Box.createGlue(), JideBoxLayout.VARY);
        return panel;
    }

    public String getDemoFolder() {
        return "G6. ColorComboBoxes";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new ColorChooserDemo());
    }

    private ColorComboBox createColorComboBox() {
        final ColorComboBox colorComboBox = new ColorComboBox();
        colorComboBox.setSelectedColor(Color.black);
        colorComboBox.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                Color selectedColor = colorComboBox.getSelectedColor();
                _messageLabel.setText("Selected color: " + ObjectConverterManager.toString(selectedColor));
            }
        });
        return colorComboBox;
    }

    private JPanel createOptionsPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new JideBoxLayout(panel, BoxLayout.Y_AXIS));
        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        String[] colorModes = new String[]{
                "15 Color Palette",
                "40 Color Palette",
                "216 Color Palette",
                "16 Gray Scale",
                "102 Gray Scale",
                "256 Gray Scale"
        };
        final JComboBox comboBox = new JComboBox(colorModes);
        comboBox.setSelectedIndex(_colorComboBox.getColorMode());
        comboBox.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (e.getStateChange() == ItemEvent.SELECTED) {
                    _colorComboBox.setColorMode(comboBox.getSelectedIndex());
                }
            }
        });

        panel.add(new JLabel("Set Color Mode"));
        panel.add(Box.createVerticalStrut(6), JideBoxLayout.FIX);
        panel.add(comboBox);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);
        panel.add(new JLabel("Additional Buttons"));
        panel.add(Box.createVerticalStrut(6), JideBoxLayout.FIX);
        final JCheckBox allowNone = new JCheckBox("Allow \"None\"");
        allowNone.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _colorComboBox.setAllowDefaultColor(allowNone.isSelected());
            }
        });
        panel.add(allowNone);
        allowNone.setSelected(true);
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        final JCheckBox allowMore = new JCheckBox("Allow \"More Color\"");
        allowMore.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _colorComboBox.setAllowMoreColors(allowMore.isSelected());
            }
        });
        allowMore.setSelected(true);
        panel.add(allowMore);
        panel.add(Box.createGlue(), JideBoxLayout.VARY);

        return panel;
    }
}
