/*
 * @(#)${NAME}.java
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.docking.DockContext;
import com.jidesoft.docking.DockableFrame;
import com.jidesoft.status.ProgressStatusBarItem;
import com.jidesoft.swing.JideButton;
import com.jidesoft.swing.JideScrollPane;
import com.jidesoft.swing.JideToggleButton;
import com.jidesoft.swing.SearchableUtils;
import com.jidesoft.tree.TreeUtils;
import com.jidesoft.utils.ProductNames;

import javax.swing.*;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.Position;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Arrays;
import java.util.Vector;

/**
 */
class DemosDockableFrame extends DockableFrame {
    private Vector _demoes = new Vector();
    private int _order; // 0 or 1

    static final String[] DEMOS = new String[]{
            // Components
            "DocumentPaneDemo",
            "StatusBarDemo",
            "CollapsiblePaneDemo",
            "FloorTabbedPaneDemo",
            "FastGradientPaintDemo",
            "AlertDemo",
            "SearchableDemo",
            "SearchableBarDemo",
            "AutoCompletionDemo",
            "IntelliHintsDemo",
            "PopupDemo",
            "JideButtonDemo",
            "JideSplitButtonDemo",
            "JideSplitPaneDemo",
            "StyledLabelDemo",
            "StyledLabelBuilderDemo",
            "FolderChooserDemo",
            "RangeSliderDemo",
            "JideTabbedPaneDemo",
            "JideScrollPaneDemo",
            "CheckBoxListDemo",
            "CheckBoxTreeDemo",
            "AutoResizingTextAreaDemo",
            "IndeterminateProgressBarDemo",
            "JideBorderLayoutDemo",
            "JideBoxLayoutDemo",
            "BorderDemo",

            // Dock
            "DockDemo",
            "TwoDockDemo",
            "InitialLayoutDockDemo",
            "HeavyweightDockDemo",

            // Action
            "SampleVsnetDemo",
            "SampleWordDemo",
            "ThreeActionDemo",

            // Grids
            "PropertyPaneDemo",
            "SortableTableDemo",
            "SortableListDemo",
            "SortableTreeDemo",
            "LargeSortableTableDemo",
            "FilterableTableDemo",
            "QuickFilterTableDemo",
            "QuickFilterListDemo",
            "QuickFilterTreeDemo",
            "TradingHierarchicalTableDemo",
            "ProgramHierarchicalTableDemo",
            "HierarchicalTableDemo",
            "FileSystemTreeTableDemo",
            "DefaultSpanTableDemo",
            "AbstractSpanTableDemo",
            "AbstractStyleTableDemo",
            "ComboBoxDemo",
            "DateComboBoxDemo",
            "DateChooserDemo",
            "CalendarViewerDemo",
            "ColorChooserDemo",
            "DirectionChooserDemo",
            "DirectionPropertyPaneDemo",
            "ConverterDemo",
            "JideTableDemo",
            "NonContiguousTableDemo",
            "TableScrollPaneDemo",
            "TreeTableScrollPaneDemo",
            "TableSplitPaneDemo",
            "ResizableTableDemo",
            "TableUtilsDemo",
            "DateSpinnerDemo",
            "ImagePreviewListDemo",
            "IPTextFieldDemo",

            // Dialogs
            "WizardDemo",
            "OptionsDialogDemo",
            "TipsOfTheDayDemo",
            "ButtonPanelDemo",
            "BannerPanelDemo",
            "StandardDialogDemo",
            "JideOptionPaneDemo",

            // Shortcut
            "ShortcutEditorDemo"
    };

    private JTree _tree;
    private JButton _openButton;

    public DemosDockableFrame(String key) {
        super(key, DemoIconsFactory.getImageIcon(DemoIconsFactory.Frame.DEMO));
        getContext().setInitSide(DockContext.DOCK_SIDE_WEST);
        setPreferredSize(new Dimension(200, 200));
        loadDemos();
        initComponents();
    }

    protected JideButton createButton(Action action) {
        JideButton button = new JideButton(action);
        button.setMargin(new Insets(0, 0, 0, 0));
        button.setPreferredSize(new Dimension(24, 22));
        button.setRequestFocusEnabled(false);
        return button;
    }

    protected JideButton createToggleButton(Action action) {
        JideButton button = new JideToggleButton(action);
        button.setMargin(new Insets(0, 0, 0, 0));
        button.setPreferredSize(new Dimension(24, 22));
        button.setRequestFocusEnabled(false);
        return button;
    }

    protected JComponent createToolBar() {
        JToolBar toolBar = new JToolBar();
        toolBar.setBorder(BorderFactory.createEmptyBorder(2, 0, 0, 0));
        toolBar.setFloatable(false);
        toolBar.setRollover(true);

        _openButton = createButton(new AbstractAction("", DemoIconsFactory.getImageIcon(DemoIconsFactory.Tree.FOLDER_OPENED)) {
            public void actionPerformed(ActionEvent e) {
                openSelectedDemo();
            }
        });
        _openButton.setToolTipText("Open the selected demo");
        _openButton.setEnabled(false);
        toolBar.add(_openButton);

        toolBar.addSeparator();

        JButton categoricViewButton = createToggleButton(new AbstractAction("", DemoIconsFactory.getImageIcon(DemoIconsFactory.Tree.TREE)) {
            public void actionPerformed(ActionEvent e) {
                _order = 0;
                TreeModel model = reloadTreeModel(_order);
                _tree.setModel(model);
                expandAll(model);
            }
        });
        categoricViewButton.setToolTipText("Categorizing by products");
        toolBar.add(categoricViewButton);

        JButton alpheticViewButton = createToggleButton(new AbstractAction("", DemoIconsFactory.getImageIcon(DemoIconsFactory.Tree.SORT)) {
            public void actionPerformed(ActionEvent e) {
                _order = 1;
                TreeModel model = reloadTreeModel(_order);
                _tree.setModel(model);
            }
        });
        alpheticViewButton.setToolTipText("Sort demo by alphebetic order");
        toolBar.add(alpheticViewButton);

        if (_order == 0) {
            categoricViewButton.setSelected(true);
        }
        else {
            alpheticViewButton.setSelected(true);
        }

        ButtonGroup group = new ButtonGroup();
        group.add(categoricViewButton);
        group.add(alpheticViewButton);

        return toolBar;
    }

    public void openSelectedDemo() {
        RootPaneContainer rootPaneContainer = getDockingManager().getRootPaneContainer();
        TreePath path = _tree.getSelectionPath();
        if (path != null) {
            Object node = path.getLastPathComponent();
            openDemo(rootPaneContainer, node);
        }
    }


    protected void initComponents() {
        TreeModel model = reloadTreeModel(_order);
        _tree = new JTree(model) {
            public TreePath getNextMatch(String prefix, int startingRow, Position.Bias bias) {
                return null;
            }
        };
        _tree.registerKeyboardAction(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                openSelectedDemo();
            }
        }, KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), JComponent.WHEN_FOCUSED);
        SearchableUtils.installSearchable(_tree);
        _tree.setCellRenderer(new DemoCellRenderer());
        _tree.setRootVisible(false);
        TreeUtils.expandAll(_tree);

        _tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                updateOpenButton();
            }
        });
        updateOpenButton();
        _tree.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (SwingUtilities.isLeftMouseButton(e) && e.getClickCount() == 2) {
                    RootPaneContainer rootPaneContainer = getDockingManager().getRootPaneContainer();
                    if (rootPaneContainer instanceof JideDemos) {
                        TreePath path = _tree.getSelectionPath();
                        if (path != null) {
                            Object node = path.getLastPathComponent();
                            openDemo(rootPaneContainer, node);
                        }
                    }
                }
            }
        });

        JPanel panel = new JPanel(new BorderLayout(2, 2));
        panel.setBorder(BorderFactory.createEmptyBorder(0, 3, 2, 3));
        _tree.setBorder(BorderFactory.createEmptyBorder(0, 3, 2, 2));
        panel.add(createToolBar(), BorderLayout.BEFORE_FIRST_LINE);
        JideScrollPane scrollPane = new JideScrollPane(_tree);
        scrollPane.setRequestFocusEnabled(false);
        scrollPane.setFocusable(false);
        panel.add(scrollPane, BorderLayout.CENTER);
        getContentPane().add(panel);
    }

    private void updateOpenButton() {
        if (_openButton == null) {
            return;
        }

        if (_tree.getSelectionPath() != null) {
            _openButton.setEnabled(true);
        }
        else {
            _openButton.setEnabled(false);
        }
    }

    private void expandAll(TreeModel model) {
        if (_order == 0) {
            for (int i = 0; i < ((ComparableTreeNode) model.getRoot()).getChildCount(); i++) {
                _tree.expandPath(new TreePath(new Object[]{model.getRoot(),
                        ((ComparableTreeNode) model.getRoot()).getChildAt(i)}));
            }
        }
    }

    /**
     * Loads tree into TreeModel
     *
     * @param order
     */
    private TreeModel reloadTreeModel(int order) {
        ComparableTreeNode root = new ComparableTreeNode("JIDE Products");
        if (order == 0) {
            ComparableTreeNode nodeDock = new ComparableTreeNode(ProductNames.PRODUCT_NAME_DOCK);
            ComparableTreeNode nodeAction = new ComparableTreeNode(ProductNames.PRODUCT_NAME_ACTION);
            ComparableTreeNode nodeComponents = new ComparableTreeNode(ProductNames.PRODUCT_NAME_COMPONENTS);
            ComparableTreeNode nodeGrids = new ComparableTreeNode(ProductNames.PRODUCT_NAME_GRIDS);
            ComparableTreeNode nodeDialogs = new ComparableTreeNode(ProductNames.PRODUCT_NAME_DIALOGS);
            ComparableTreeNode nodeShortcut = new ComparableTreeNode(ProductNames.PRODUCT_NAME_SHORTCUT);
            root.add(nodeDock);
            root.add(nodeAction);
            root.add(nodeComponents);
            root.add(nodeGrids);
            root.add(nodeDialogs);
            root.add(nodeShortcut);

            for (int i = 0; i < _demoes.size(); i++) {
                ComparableTreeNode node = (ComparableTreeNode) _demoes.get(i);
                if (node.getUserObject() instanceof Demo) {
                    Demo demo = (Demo) node.getUserObject();
                    Object product = demo.getProduct();
                    if (ProductNames.PRODUCT_NAME_DOCK.equals(product)) {
                        nodeDock.add(node);
                    }
                    else if (ProductNames.PRODUCT_NAME_ACTION.equals(product)) {
                        nodeAction.add(node);
                    }
                    else if (ProductNames.PRODUCT_NAME_COMPONENTS.equals(product)) {
                        nodeComponents.add(node);
                    }
                    else if (ProductNames.PRODUCT_NAME_GRIDS.equals(product)) {
                        nodeGrids.add(node);
                    }
                    else if (ProductNames.PRODUCT_NAME_DIALOGS.equals(product)) {
                        nodeDialogs.add(node);
                    }
                    else if (ProductNames.PRODUCT_NAME_SHORTCUT.equals(product)) {
                        nodeShortcut.add(node);
                    }
                }
            }
        }
        else {
            Object[] demoes = _demoes.toArray();
            Arrays.sort(demoes);
            for (int i = 0; i < demoes.length; i++) {
                ComparableTreeNode node = (ComparableTreeNode) demoes[i];
                root.add(node);
            }
        }

        return new DefaultTreeModel(root);
    }

    /**
     * Loads all demos into a list of tree nodes.
     */
    private void loadDemos() {
        int loaded = 0;
        _demoes.clear();
        for (int i = 0; i < DEMOS.length; i++) {
            String demoClasName = DEMOS[i];
            try {
                Class demoClass = getClass().getClassLoader().loadClass(demoClasName);
                Demo demo = (Demo) demoClass.newInstance();
                ComparableTreeNode demoNode = new ComparableTreeNode(demo);
                _demoes.add(demoNode);
                loaded++;
            }
            catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
            catch (InstantiationException e) {
                e.printStackTrace();
            }
            catch (IllegalAccessException e) {
                e.printStackTrace();
            }
            ((ProgressStatusBarItem) JideDemos.getStatusBar().getItemByName("Status")).setStatus("Total " + DEMOS.length + " demos; "
                    + ((loaded == DEMOS.length) ? "all" : "" + loaded) + " of them are loaded successfully.");
        }
    }

    private void openDemo(RootPaneContainer rootPaneContainer, Object node) {
        Object object = ((ComparableTreeNode) node).getUserObject();
        if (object instanceof Demo) {
            ((JideDemos) rootPaneContainer).openDemo(((Demo) object));
        }
    }

    class ComparableTreeNode extends DefaultMutableTreeNode implements Comparable {
        public ComparableTreeNode() {
        }

        public ComparableTreeNode(Object userObject) {
            super(userObject);
        }

        public ComparableTreeNode(Object userObject, boolean allowsChildren) {
            super(userObject, allowsChildren);
        }

        public int compareTo(Object o) {
            return getUserObject().toString().compareTo(((ComparableTreeNode) o).getUserObject().toString());
        }
    }
}
