/*
 * @(#)DirectionCellEditor.java 4/6/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.grid.ContextSensitiveCellEditor;
import com.jidesoft.grid.EditorContext;
import com.jidesoft.swing.JideSwingUtilities;

import javax.swing.*;
import javax.swing.table.TableCellEditor;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 * A direction cell editor.
 */
public class DirectionCellEditor extends ContextSensitiveCellEditor implements TableCellEditor, ItemListener {

    public final static EditorContext CONTEXT = new EditorContext("Direction");

    private DirectionComboBox _comboBox;

    /**
     * Creates a FileNameCellEditor.
     */
    public DirectionCellEditor() {
        _comboBox = new DirectionComboBox();
        _comboBox.setBorder(BorderFactory.createEmptyBorder());
    }

    /**
     * Gets the value of the cell editor.
     *
     * @return the value in this cell editor.
     */
    public Object getCellEditorValue() {
        _comboBox.setSelectedItem(_comboBox.getEditor().getItem());
        return new Integer(_comboBox.getSelectedDirection());
    }

    public Component getTableCellEditorComponent(JTable table, Object value,
                                                 boolean isSelected,
                                                 int row, int column) {
        if (table != null) {
            JideSwingUtilities.installColorsAndFont(_comboBox, table.getBackground(), table.getForeground(), table.getFont());
        }
        _comboBox.setSelectedItem(value, false);
        _comboBox.setConverterContext(getConverterContext());
        _comboBox.addItemListener(this);
        return _comboBox;
    }

    public boolean stopCellEditing() {
        _comboBox.setPopupVisible(false);
        return super.stopCellEditing();
    }

    public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
            _comboBox.removeItemListener(this);
            stopCellEditing();
        }
    }
}