/*
 * @(#)DocumentPaneDemo.java 4/3/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.dialog.ButtonPanel;
import com.jidesoft.document.DocumentComponent;
import com.jidesoft.document.DocumentPane;
import com.jidesoft.icons.JideIconsFactory;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JideTabbedPane;

import javax.swing.*;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

/**
 * Demoed Component: {@link DocumentPane}
 * <br>
 * Required jar files: jide-common.jar, jide-components.jar
 * <br>
 * Required L&F: Jide L&F extension required
 */
public class DocumentPaneDemo extends AbstractDemo {
    public DocumentPane _documentPane;

    public DocumentPaneDemo() {
    }

    public String getName() {
        return "DocumentPane Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_COMPONENTS;
    }

    public String getDescription() {
        return "DocumentPane is tabbed document interface (or TDI in short) implementation. It supports viewing multiple documents using tabbed pane, viewing two ore more documents at the same time, view several different views of the same documents, document open/close/activate/deactivate events, drag-n-drop organizing documents and many other advanced features. It's an ideal component to replace traditional MDI implentation using JDesktopPane.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.document.DocumentPane\n" +
                "com.jidesoft.document.DocumentComponent";
    }

    public Component getOptionsPanel() {
        ButtonPanel buttonPanel = new ButtonPanel(SwingConstants.TOP);
        buttonPanel.addButton(new JButton(new AbstractAction("Reset Documents Layout") {
            public void actionPerformed(ActionEvent e) {
                java.util.List list = new ArrayList();
                list.add(new DocumentComponent(createMultiViewDocument("EULA.htm"),
                        "License", "License", JideIconsFactory.getImageIcon(JideIconsFactory.FileType.HTML)));
                list.add(new DocumentComponent(new JScrollPane(DocumentPaneDemo.createTextArea("Readme.txt")),
                        "Readme", "Readme",
                        JideIconsFactory.getImageIcon(JideIconsFactory.FileType.TEXT)));
                list.add(new DocumentComponent(new JScrollPane(DocumentPaneDemo.createTextArea("DocumentPaneDemo.java")),
                        "DocumentPaneDemo.java", "C:\\Projects\\JideSoft\\Source Code\\DocumentPaneDemo.java",
                        JideIconsFactory.getImageIcon(JideIconsFactory.FileType.JAVA)));
                _documentPane.setOpenedDocuments(list);
                _documentPane.getLayoutPersistence().resetToDefault();
            }
        }));
        return buttonPanel;
    }

    public Component getDemoPanel() {
        _documentPane = createDocumentTabs();
//        _documentPane.setMaximumGroupCount(3);
        _documentPane.setGroupsAllowed(true);
        _documentPane.setTabbedPaneCustomizer(new DocumentPane.TabbedPaneCustomizer() {
            public void customize(final JideTabbedPane tabbedPane) {
                tabbedPane.setUseDefaultShowCloseButtonOnTab(false);
                tabbedPane.setShowCloseButtonOnTab(true);
            }
        });
        _documentPane.setPreferredSize(new Dimension(500, 400));
        return _documentPane;
    }

    public String getDemoFolder() {
        return "C1. DocumentPane";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new DocumentPaneDemo());
    }

    private DocumentPane createDocumentTabs() {
        DocumentPane pane = new DocumentPane();
        java.util.List list = new ArrayList();
        list.add(new DocumentComponent(createMultiViewDocument("EULA.htm"),
                "License", "License", JideIconsFactory.getImageIcon(JideIconsFactory.FileType.HTML)));
        DocumentComponent document = new DocumentComponent(new JScrollPane(DocumentPaneDemo.createTextArea("Readme.txt")),
                "Readme", "Readme",
                JideIconsFactory.getImageIcon(JideIconsFactory.FileType.TEXT));
        list.add(document);
        list.add(new DocumentComponent(new JScrollPane(DocumentPaneDemo.createTextArea("DocumentPaneDemo.java")),
                "DocumentPaneDemo.java", "C:\\Projects\\JideSoft\\Source Code\\DocumentPaneDemo.java",
                JideIconsFactory.getImageIcon(JideIconsFactory.FileType.JAVA)));
        pane.setOpenedDocuments(list);
        pane.getLayoutPersistence().setProfileKey("documents");
        pane.getLayoutPersistence().loadLayoutData();
        return pane;
    }

    private static JComponent createMultiViewDocument(String fileName) {
        JideTabbedPane pane = new JideTabbedPane(JideTabbedPane.BOTTOM);
        pane.setBoxStyleTab(true);
        pane.addTab("Design", JideIconsFactory.getImageIcon(JideIconsFactory.View.DESIGN), new JScrollPane(createHtmlArea(fileName)));
        pane.addTab("HTML", JideIconsFactory.getImageIcon(JideIconsFactory.View.HTML), new JScrollPane(createTextArea(fileName)));
        return pane;
    }

    public static JTextArea createTextArea(String fileName) {
        JTextArea area = new JTextArea();
        Document doc = new PlainDocument();
        try {
            // try to start reading
            InputStream in = DocumentPaneDemo.class.getResourceAsStream(fileName);
            if (in != null) {
                byte[] buff = new byte[4096];
                int nch;
                while ((nch = in.read(buff, 0, buff.length)) != -1) {
                    doc.insertString(doc.getLength(), new String(buff, 0, nch), null);
                }
                area.setDocument(doc);
            }
            else {
                area.setText("Copy Readme.txt and DocumentPaneDemo.java into the class output directory");
            }
        }
        catch (FileNotFoundException e) {
        }
        catch (IOException e) {
        }
        catch (BadLocationException e) {
        }
        return area;
    }

    public static Component createHtmlArea(String fileName) {
        JEditorPane area = new JEditorPane();
        try {
            area.setPage(DocumentPaneDemo.class.getResource(fileName));
        }
        catch (IOException e) {
            area.setText("Copy EULA.html into the class output directory");
        }
        return area;
    }
}
