/*
 * @(#)HierarchicalTableDemo.java 2/14/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.dialog.ButtonPanel;
import com.jidesoft.grid.*;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.utils.Lm;
import com.jidesoft.utils.SystemInfo;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableCellRenderer;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseWheelListener;
import java.util.Arrays;

/**
 * Demoed Component: {@link com.jidesoft.grid.HierarchicalTable}
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: Jide L&F extension required
 */
public class HierarchicalTableDemo extends AbstractDemo {

    protected static final Color BG1 = new Color(232, 237, 230);
    protected static final Color BG2 = new Color(243, 234, 217);
    protected static final Color BG3 = new Color(214, 231, 247);

    private HierarchicalTable _table;
    private TableModel _productsTableModel;

    private ListSelectionModelGroup _group = new ListSelectionModelGroup();

    public HierarchicalTableDemo() {
    }

    public String getName() {
        return "HierarchicalTable (Nested Table) Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_GRIDS;
    }

    public String getDescription() {
        return "This is a demo of HierarchicalTable. HierarchicalTable is a JTable which supports nested components for each row.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.grid.HierarchicalTable\n" +
                "com.jidesoft.grid.HierarchicalTableModel\n" +
                "com.jidesoft.grid.ListSelectionModelGroup";
    }

    public Component getOptionsPanel() {
        ButtonPanel buttonPanel = new ButtonPanel(SwingConstants.TOP);
        JButton button = new JButton(new AbstractAction("Insert a row") {
            public void actionPerformed(ActionEvent e) {
                ((DefaultTableModel) _productsTableModel).insertRow(0, new Object[]{"JIDE Action Framework", "Feature drag-n-dropable toolbar/menu bar components"});
            }
        });
        button.setRequestFocusEnabled(false);
        buttonPanel.addButton(button);

        button = new JButton(new AbstractAction("Delete selected rows") {
            public void actionPerformed(ActionEvent e) {
                int[] rows = _table.getSelectedRows();
                for (int i = 0; i < rows.length; i++) {
                    int row = rows[i];
                    rows[i] = TableModelWrapperUtils.getActualRowAt(_table.getModel(), row, HierarchicalTableModel.class);
                }
                Arrays.sort(rows);
                for (int i = rows.length - 1; i >= 0; i--) {
                    int row = rows[i];
                    ((DefaultTableModel) _productsTableModel).removeRow(row);
                }
            }
        });
        button.setRequestFocusEnabled(false);
        buttonPanel.addButton(button);

        button = new JButton(new AbstractAction("Update a cell") {
            public void actionPerformed(ActionEvent e) {
                _productsTableModel.setValueAt("Collection of useful components include tabbed document interface, status bar, floor tabbed pane and collapsible pane.", 1, 1);
            }
        });
        button.setRequestFocusEnabled(false);
        buttonPanel.addButton(button);
        return buttonPanel;
    }

    public Component getDemoPanel() {
        _table = createTable();
        return new JScrollPane(_table);
    }

    public String getDemoFolder() {
        return "G8. HierarchicalTable";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new HierarchicalTableDemo());
    }

    // create property table
    private HierarchicalTable createTable() {
        _productsTableModel = new ProductTableModel();
        HierarchicalTable table = new HierarchicalTable();
        table.setModel(_productsTableModel);
        table.setBackground(BG1);
        table.setName("Product Table");
        table.getColumnModel().getColumn(0).setPreferredWidth(100);
        table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        table.setComponentFactory(new HierarchicalTableComponentFactory() {
            public Component createChildComponent(HierarchicalTable table, Object value, int row) {
                if (value == null) {
                    return new JPanel();
                }
                TableModel model = ((TableModel) value);
                if (model instanceof HierarchicalTableModel) {
                    HierarchicalTable table1 = new HierarchicalTable(model);
                    table1.setBackground(BG2);
                    table1.setOpaque(true);
                    table1.setName("Detail Table");
                    table1.setComponentFactory(new HierarchicalTableComponentFactory() {
                        public Component createChildComponent(HierarchicalTable table, Object value, int row) {
                            if (value instanceof TableModel) {
                                TableModel model = ((TableModel) value);
                                if (model != null) {
                                    SortableTable sortableTable = new SortableTable(model);
                                    HierarchicalTableDemo.FitScrollPane pane = new HierarchicalTableDemo.FitScrollPane(sortableTable);
                                    sortableTable.setBackground(BG3);
                                    Dimension size = sortableTable.getPreferredSize();
                                    pane.setPreferredSize(new Dimension(size.width, size.height + 24));
                                    _group.add(sortableTable.getSelectionModel());
                                    return new TreeLikeHierarchicalPanel(pane);
                                }
                            }
                            return null;
                        }

                        public void destroyChildComponent(HierarchicalTable table, Component component, int row) {
                            if (component instanceof JTable) {
                                _group.remove(((JTable) component).getSelectionModel());
                            }
                        }
                    });
                    _group.add(table1.getSelectionModel());
                    return new TreeLikeHierarchicalPanel(new HierarchicalTableDemo.FitScrollPane(table1));
                }
                else {
                    SortableTable sortableTable = new SortableTable(model);
                    sortableTable.setBackground(BG2);
                    _group.add(sortableTable.getSelectionModel());
                    return new TreeLikeHierarchicalPanel(new HierarchicalTableDemo.FitScrollPane(sortableTable));
                }
            }

            public void destroyChildComponent(HierarchicalTable table, Component component, int row) {
                if (component instanceof JTable) {
                    _group.remove(((JTable) component).getSelectionModel());
                }
            }
        });
        _group.add(table.getSelectionModel());
        return table;
    }


    static String[] DETAIL_COLUMNS = new String[]{"Feature", "Description"};
    static String[] PRODUCT_COLUMNS = new String[]{"Name", "Description"};

    static String[][] JIDE_PRODUCTS = new String[][]{
        new String[]{"JIDE Docking Framework", "Swing-based GUI Framework which enables drag-n-drop dockable windows"},
        new String[]{"JIDE Components", "Collection of useful components"},
        new String[]{"JIDE Grids", "Collection of JTable related components"},
        new String[]{"JIDE Dialogs", "Collection of JDialog related components", }
    };

    static String[][] JIDE_DOCK = new String[][]{
        new String[]{"Drag-n-drop", ""},
        new String[]{"Autohide Windows", ""},
        new String[]{"Nested Floating Windows", ""},
        new String[]{"Persist Layouts", ""}
    };

    static String[][] JIDE_ACTION = new String[][]{
        new String[]{"Drag-n-drop command bars", ""},
        new String[]{"Floating command bars", ""},
        new String[]{"Persist Layouts", ""}
    };

    static String[][] JIDE_COMP = new String[][]{
        new String[]{"DocumentPane", "Drag-n-drop tabbed document interface as you see in modern IDEs etc"},
        new String[]{"Status Bar", "Full customizable status bar that you can use to display status, progress, time and other information at the bottom of your application"},
        new String[]{"FloorTabbedPane", "Shortcut Bar, just like in Outlook"},
        new String[]{"CollapsiblePane", "Task Bar which you can find in Windows XP"}
    };

    static String[][] JIDE_GRIDS = new String[][]{
        new String[]{"PropertyTable", "A two-column JTable used to display properties of any object"},
        new String[]{"SortableTable", "A JTable which allows multiple-column sorting"},
        new String[]{"FilterTableModel", "A table model which allows filters on each column"},
        new String[]{"HierarchicalTable", "A JTable which allows hierarchical display of components on each row"},
        new String[]{"ComboBox", "A collection of ComboBoxes"},
        new String[]{"CellEditors", "A collection of CellEditor for many types of objects"},
        new String[]{"Converters", "A list of converters which can convert String to/from any other types"},
        new String[]{"TableUtils", "A utility class which contains several useful methods for JTable"}
    };

    static String[][] JIDE_COMBOBOXES = new String[][]{
        new String[]{"ColorComboBox", "A ComboBox to choose color"},
        new String[]{"DateComboBox", "A ComboBox to choose date"},
        new String[]{"MonthComboBox", "A ComboBox to choose month"},
        new String[]{"FileChooserComboBox", "A ComboBox to choose file"},
        new String[]{"FontComboBox", "A ComboBox to choose font"},
        new String[]{"ListComboBox", "A ComboBox to choose an item from a list"},
        new String[]{"BooleanComboBox", "A ComboBox to choose true or false"},
        new String[]{"StringArrayComboBox", "A ComboBox to choose String[]"}
    };

    static String[][] JIDE_CELLEDITORS = new String[][]{
        new String[]{"ColorCellEditor", "A cell editor to choose a color"},
        new String[]{"DateCellEditor", "A cell editor to choose a date"},
        new String[]{"MonthCellEditor", "A cell editor to choose a month"},
        new String[]{"FileNameCellEditor", "A cell editor to choose file name"},
        new String[]{"DoubleCellEditor", "A cell editor to edit double"},
        new String[]{"IntegerCellEditor", "A cell editor to edit int"},
        new String[]{"NumberCellEditor", "A cell editor to edit number"},
        new String[]{"ListComboBoxCellEditor", "A cell editor to choose an item from a list"},
        new String[]{"BooleanCellEditor", "A cell editor to choose true or false"},
        new String[]{"FontNameCellEditor", "A cell editor to choose a font name"},
        new String[]{"StringArrayCellEditor", "A cell editor to choose a String[]"}
    };

    static class GridsProductTableModel extends DefaultTableModel implements HierarchicalTableModel {
        public GridsProductTableModel() {
            super(JIDE_GRIDS, DETAIL_COLUMNS);
        }

        public boolean isCellEditable(int row, int column) {
            return false;
        }

        public boolean hasChild(int row) {
            Object name = getValueAt(row, 0);
            if ("CellEditors".equals(name) || "ComboBox".equals(name)) {
                return true;
            }
            else {
                return false;
            }
        }

        public boolean isHierarchical(int row) {
            return true;
        }

        public boolean isExpandable(int row) {
            return true;
        }

        public Object getChildValueAt(int row) {
            TableModel model = null;
            Object name = getValueAt(row, 0);
            if ("ComboBox".equals(name)) {
                model = new DefaultTableModel(JIDE_COMBOBOXES, DETAIL_COLUMNS) {
                    public boolean isCellEditable(int row, int column) {
                        return false;
                    }
                };
            }
            else if ("CellEditors".equals(name)) {
                model = new DefaultTableModel(JIDE_CELLEDITORS, DETAIL_COLUMNS) {
                    public boolean isCellEditable(int row, int column) {
                        return false;
                    }
                };
            }
            return model;
        }
    }

    static String[][] JIDE_DIALOGS = new String[][]{
        new String[]{"StandardDialog", "A common dialog class which implements some common features that all dialogs will need"},
        new String[]{"Lazy-loading Page", "Lazy loading panel with page event (open, closing, closed etc) support"},
        new String[]{"Wizard", "Wizard component which support both Microsoft Wizard 97 standard and Java L&F standard"},
        new String[]{"MultiplePageDialog", "A dialog contains many pages which can be used as options dialog or user preference dialog"},
        new String[]{"ButtonPanel", "Layout buttons in different order based OS convention"},
        new String[]{"BannerPanel", "A banner panel mainly for decoration purpose"},
        new String[]{"Tips of the Day Dialog", "A dialog shows tips of the day"}
    };

    static class ProductTableModel extends DefaultTableModel implements HierarchicalTableModel {

        public ProductTableModel() {
            super(JIDE_PRODUCTS, PRODUCT_COLUMNS);
        }

        public boolean isCellEditable(int row, int column) {
            return false;
        }

        public boolean hasChild(int row) {
            return true;
        }

        public boolean isExpandable(int row) {
            return true;
        }

        public boolean isHierarchical(int row) {
            return true;
        }

        public Object getChildValueAt(int row) {
            TableModel model = null;
            Object name = getValueAt(row, 0);
            if ("JIDE Docking Framework".equals(name)) {
                model = new DefaultTableModel(JIDE_DOCK, DETAIL_COLUMNS) {
                    public boolean isCellEditable(int row, int column) {
                        return false;
                    }
                };
            }
            else if ("JIDE Components".equals(name)) {
                model = new DefaultTableModel(JIDE_COMP, DETAIL_COLUMNS) {
                    public boolean isCellEditable(int row, int column) {
                        return false;
                    }
                };
            }
            else if ("JIDE Grids".equals(name)) {
                model = new HierarchicalTableDemo.GridsProductTableModel();
            }
            else if ("JIDE Dialogs".equals(name)) {
                model = new DefaultTableModel(JIDE_DIALOGS, DETAIL_COLUMNS) {
                    public boolean isCellEditable(int row, int column) {
                        return false;
                    }
                };
            }
            else if ("JIDE Action Framework".equals(name)) {
                model = new DefaultTableModel(JIDE_ACTION, DETAIL_COLUMNS) {
                    public boolean isCellEditable(int row, int column) {
                        return false;
                    }
                };
            }
            return model;
        }
    }

    static class FitScrollPane extends JScrollPane implements ComponentListener {
        public FitScrollPane() {
            initScrollPane();
        }

        public FitScrollPane(Component view) {
            super(view);
            initScrollPane();
        }

        public FitScrollPane(Component view, int vsbPolicy, int hsbPolicy) {
            super(view, vsbPolicy, hsbPolicy);
            initScrollPane();
        }

        public FitScrollPane(int vsbPolicy, int hsbPolicy) {
            super(vsbPolicy, hsbPolicy);
            initScrollPane();
        }

        private void initScrollPane() {
            setBorder(BorderFactory.createLineBorder(Color.GRAY));
            setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
            setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_NEVER);
            getViewport().getView().addComponentListener(this);
            removeMouseWheelListeners();
        }

        // remove MouseWheelListener as there is no need for it in FitScrollPane.
        private void removeMouseWheelListeners() {
            MouseWheelListener[] listeners = getMouseWheelListeners();
            for (int i = 0; i < listeners.length; i++) {
                MouseWheelListener listener = listeners[i];
                removeMouseWheelListener(listener);
            }
        }

        public void updateUI() {
            super.updateUI();
            removeMouseWheelListeners();
        }

        public void componentResized(ComponentEvent e) {
            setSize(getSize().width, getPreferredSize().height);
            if (Lm.HG_DEBUG) {
                System.out.println("FitScrollPane resized " + this.getClass().getName());
            }
        }

        public void componentMoved(ComponentEvent e) {
        }

        public void componentShown(ComponentEvent e) {
        }

        public void componentHidden(ComponentEvent e) {
        }

        public Dimension getPreferredSize() {
            getViewport().setPreferredSize(getViewport().getView().getPreferredSize());
            return super.getPreferredSize();
        }
    }
}
