/*
 * @(#)IntelliHintsDemo.java 7/24/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.hints.AbstractIntelliHints;
import com.jidesoft.hints.AbstractListIntelliHints;
import com.jidesoft.hints.FileIntelliHints;
import com.jidesoft.hints.ListDataIntelliHints;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JideBoxLayout;

import javax.swing.*;
import java.awt.*;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Vector;
import java.util.zip.GZIPInputStream;

/**
 * Demoed Component: {@link com.jidesoft.hints.IntelliHints}.
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: Jide L&F extension required
 */
public class IntelliHintsDemo extends AbstractDemo {

    public IntelliHintsDemo() {
    }

    public String getName() {
        return "IntelliHints Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_COMPONENTS;
    }

    public String getDescription() {
        return "This is a demo of IntelliHints components. IntelliHints can display a hint popup in a text field or text area " +
                "so that user can pick a hint directly while typing.\n" +
                "\nYou can start to type in those text fields or text area to see how it works. " +
                "At any time, if you want to see whether there are hints available, you can press DOWN key " +
                "in text field or CTRL+SPACE in text area.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.hints.IntelliHints\n" +
                "com.jidesoft.hints.AbstractIntelliHints\n" +
                "com.jidesoft.hints.AbstractListIntelliHints\n" +
                "com.jidesoft.hints.FileIntelliHints\n" +
                "com.jidesoft.hints.ListDataIntelliHints";
    }

    protected java.util.List readUrls() throws IOException {
        InputStream resource = this.getClass().getClassLoader().getResourceAsStream("url.txt.gz");
        if (resource == null) {
            return null;
        }

        InputStream in = new GZIPInputStream(resource);
        BufferedReader reader = new BufferedReader(new InputStreamReader(in, "UTF-8"));

        java.util.List list = new ArrayList();
        while (true) {
            String line = reader.readLine(); // skip first line
            if (line == null || line.length() == 0) {
                break;
            }
            list.add(line);
        }
        return list;
    }

    public Component getDemoPanel() {
        JPanel panel = new JPanel();
        panel.setLayout(new JideBoxLayout(panel, JideBoxLayout.Y_AXIS));
        panel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        String[] fontNames = GraphicsEnvironment.getLocalGraphicsEnvironment().getAvailableFontFamilyNames();
        // create file text field
        java.util.List urls = null;
        try {
            urls = readUrls();
        }
        catch (IOException e) {
            e.printStackTrace();
        }

        JTextField urlTextField = new JTextField("http://");
        ListDataIntelliHints intellihints = new ListDataIntelliHints(urlTextField, urls);
        intellihints.setCaseSensitive(false);
        panel.add(new JLabel("ListDataIntelliHints TextField for URLs: "));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(urlTextField);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create path text field
        JTextField pathTextField = new JTextField();
        FileIntelliHints intelliHints = new FileIntelliHints(pathTextField);
        intelliHints.setFolderOnly(true);
        panel.add(new JLabel("FileIntelliHints TextField for path (folders only):"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(pathTextField);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create file text field
        JTextField fileTextField = new JTextField();
        new FileIntelliHints(fileTextField);
        panel.add(new JLabel("FileIntelliHints TextField for file (files and folders):"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(fileTextField);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create file text field
        JTextArea fileTextArea = new JTextArea();
        new FileIntelliHints(fileTextArea);
        panel.add(new JLabel("FileIntelliHints TextArea for files (each line is a new file):"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(new JScrollPane(fileTextArea));
        fileTextArea.setRows(4);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        // create file text field
        JTextField fontTextField = new JTextField();
        ListDataIntelliHints listIntellihints = new ListDataIntelliHints(fontTextField, fontNames);
        listIntellihints.setCaseSensitive(false);
        panel.add(new JLabel("IntelliHints TextField to choose font name:"));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        panel.add(fontTextField);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        panel.add(new JLabel("A custom IntelliHints for prime numbers: "));
        panel.add(Box.createVerticalStrut(3), JideBoxLayout.FIX);
        JTextField textField = new JTextField();
        AbstractIntelliHints primeNumberHints = new AbstractListIntelliHints(textField) {
            protected JLabel _messageLabel;

            public JComponent createHintsComponent() {
                JPanel panel = (JPanel) super.createHintsComponent();
                _messageLabel = new JLabel();
                panel.add(_messageLabel, BorderLayout.BEFORE_FIRST_LINE);
                return panel;
            }

            // update list model depending on the data in textfield
            public boolean updateHints(Object value) {
                if (value == null) {
                    return false;
                }
                String s = value.toString();
                s = s.trim();
                if (s.length() == 0) {
                    return false;
                }
                try {
                    long l = Long.parseLong(s);
                    boolean prime = isProbablePrime(l);
                    _messageLabel.setText("");
                    if (prime) {
                        return false;
                    }
                    else {
                        Vector list = new Vector();
                        long nextPrime = l;
                        for (int i = 0; i < 10; i++) {
                            nextPrime = nextPrime(nextPrime);
                            list.add(new Long(nextPrime));
                        }
                        setListData(list);
                        _messageLabel.setText("Next 10 prime numbers:");
                        _messageLabel.setForeground(Color.DARK_GRAY);
                        return true;
                    }
                }
                catch (NumberFormatException e) {
                    setListData(new Object[0]);
                    _messageLabel.setText("Invalid long number");
                    setListData(new Object[0]);
                    _messageLabel.setForeground(Color.RED);
                    return true;
                }
            }
        };
        panel.add(textField);
        panel.add(Box.createVerticalStrut(12), JideBoxLayout.FIX);

        panel.add(Box.createGlue(), JideBoxLayout.VARY);

        return panel;
    }

    public String getDemoFolder() {
        return "B14. IntelliHints";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new IntelliHintsDemo());
    }

    public static boolean isProbablePrime(long number) {
        return new BigInteger("" + number).isProbablePrime(500);
    }

    public static long nextPrime(long lastPrime) {
        long testPrime;
        testPrime = lastPrime + 1;
        while (!isProbablePrime(testPrime)) testPrime += (testPrime % 2 == 0) ? 1 : 2;

        return testPrime;
    }

}
