/*
 * @(#)JideSplitButtonDemo.java 3/19/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JideSplitButton;
import com.jidesoft.swing.JideSwingUtilities;
import com.jidesoft.swing.JideToggleSplitButton;
import com.jidesoft.utils.SystemInfo;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 * Demoed Component: {@link com.jidesoft.swing.JideButton}
 * <br>
 * Required jar files: jide-common.jar, jide-components.jar
 * <br>
 * Required L&F: Jide L&F extension required
 */
public class JideSplitButtonDemo extends AbstractDemo {
    private JideSplitButton[] _buttons;

    public JideSplitButtonDemo() {
    }

    public String getName() {
        return "JideSplitButton Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_COMPONENTS;
    }

    public String getDescription() {
        return "JideSplitButton is button and menu combination. It has a button part just like JideButton but it also has a menu part where a popup menu could be added.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.swing.JideSplitButton\n";
    }

    public Component getDemoPanel() {
        JPanel panel = new JPanel(new GridLayout(6, 1));
        _buttons = new JideSplitButton[6];
        _buttons[0] = createJideSplitButton("Rename this file", ButtonsIconsFactory.getImageIcon(ButtonsIconsFactory.Buttons.RENAME));
        _buttons[1] = createJideSplitButton("Move this file", ButtonsIconsFactory.getImageIcon(ButtonsIconsFactory.Buttons.MOVE));
        _buttons[2] = createJideSplitButton("Copy this file", ButtonsIconsFactory.getImageIcon(ButtonsIconsFactory.Buttons.COPY));
        _buttons[3] = createJideSplitButton("Publish this file", ButtonsIconsFactory.getImageIcon(ButtonsIconsFactory.Buttons.PUBLISH));
        _buttons[4] = createJideSplitButton("Email this file", ButtonsIconsFactory.getImageIcon(ButtonsIconsFactory.Buttons.EMAIL));
        _buttons[5] = createJideSplitButton("Delete this file", ButtonsIconsFactory.getImageIcon(ButtonsIconsFactory.Buttons.DELET));

        for (int i = 0; i < _buttons.length; i++) {
            JideSplitButton button = _buttons[i];
            panel.add(button);
        }

        return JideSwingUtilities.createTopPanel(panel);
    }

    public String getDemoFolder() {
        return "B4. JideButton";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new JideSplitButtonDemo());
    }

    public Component getOptionsPanel() {
        JPanel switchPanel = new JPanel(new GridLayout(0, 1, 3, 3));

        final JRadioButton style1 = new JRadioButton("Toolbar Style");
        final JRadioButton style2 = new JRadioButton("Toolbox Style");
        final JRadioButton style3 = new JRadioButton("Flat Style");

        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(style1);
        buttonGroup.add(style2);
        buttonGroup.add(style3);

        switchPanel.add(new JLabel("Styles:"));
        switchPanel.add(style1);
        switchPanel.add(style2);
        switchPanel.add(style3);
        style1.setSelected(true);

        style1.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (style1.isSelected()) {
                    for (int i = 0; i < _buttons.length; i++) {
                        JideSplitButton button = _buttons[i];
                        button.setButtonStyle(JideSplitButton.TOOLBAR_STYLE);
                    }
                }
            }
        });
        style2.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (style2.isSelected()) {
                    for (int i = 0; i < _buttons.length; i++) {
                        JideSplitButton button = _buttons[i];
                        button.setButtonStyle(JideSplitButton.TOOLBOX_STYLE);
                    }
                }
            }
        });
        style3.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (style3.isSelected()) {
                    for (int i = 0; i < _buttons.length; i++) {
                        JideSplitButton button = _buttons[i];
                        button.setButtonStyle(JideSplitButton.FLAT_STYLE);
                    }
                }
            }
        });

        switchPanel.add(new JLabel("Options: "));

        final JCheckBox option1 = new JCheckBox("Button part enabled");
        final JCheckBox option2 = new JCheckBox("Both parts enabled");
        final JCheckBox option3 = new JCheckBox("Button part selected");
        final JCheckBox option4 = new JCheckBox("Always Dropdown");

        switchPanel.add(option1);
        switchPanel.add(option2);
        switchPanel.add(option3);
        switchPanel.add(option4);

        option1.setSelected(true);
        option1.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                for (int i = 0; i < _buttons.length; i++) {
                    JideSplitButton button = _buttons[i];
                    button.setButtonEnabled(option1.isSelected());
                }
            }
        });

        option2.setSelected(true);
        option2.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                for (int i = 0; i < _buttons.length; i++) {
                    JideSplitButton button = _buttons[i];
                    button.setEnabled(option2.isSelected());
                }
            }
        });

        option3.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                for (int i = 0; i < _buttons.length; i++) {
                    JideSplitButton button = _buttons[i];
                    button.setButtonSelected(option3.isSelected());
                }
            }
        });

        option4.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                for (int i = 0; i < _buttons.length; i++) {
                    JideSplitButton button = _buttons[i];
                    button.setAlwaysDropdown(option4.isSelected());
                }
            }
        });

        return switchPanel;
    }

    static JideSplitButton createJideSplitButton(String name, Icon icon) {
        final JideSplitButton button = new JideSplitButton(name);
        button.setIcon(icon);
        button.add(new AbstractAction("Sample Menu Item") {
            public void actionPerformed(ActionEvent e) {
            }
        });
        return button;
    }

    static JideSplitButton createJideToggleSplitButton(String name, Icon icon) {
        final JideSplitButton button = new JideToggleSplitButton(name);
        button.setIcon(icon);
        button.add(new AbstractAction("Sample Menu Item") {
            public void actionPerformed(ActionEvent e) {
            }
        });
        return button;
    }
}
