/*
 * @(#) PropertyPaneDemo.java
 * Last modified date: 2/9/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.combobox.FileChooserPanel;
import com.jidesoft.combobox.FileNameChooserComboBox;
import com.jidesoft.combobox.ListComboBox;
import com.jidesoft.combobox.PopupPanel;
import com.jidesoft.converter.*;
import com.jidesoft.grid.*;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.utils.Lm;

import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.MaskFormatter;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;

/**
 * Demoed Component: {@link PropertyPane}, {@link PropertyTable}
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: Jide L&F extension required
 */
public class PropertyPaneDemo extends AbstractDemo {

    private static PropertyTable _table;

    public PropertyPaneDemo() {
    }

    public String getDescription() {
        return "This is a demo of PropertyTable and PropertyPane. PropertyTable is a special two-column table which is ideal for showing name/value pairs. PropertyPane is a wrapper around PropertyTable with toolbar buttons to support typical operations on PropertyTable\n" +
                "\nPropertyPane is backed by PropertyTableModel. You can define mutliple levels of nested categories and still be able to show them in PropertyTable.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.grid.PropertyPane\n" +
                "com.jidesoft.grid.PropertyTable\n" +
                "com.jidesoft.grid.PropertyTableModel\n" +
                "com.jidesoft.grid.Property\n" +
                "com.jidesoft.grid.CellEditorManager\n" +
                "com.jidesoft.grid.CellRendererManager\n" +
                "com.jidesoft.converter.ObjectConverterManager";
    }

    public String getDemoFolder() {
        return "G1. PropertyPane";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new PropertyPaneDemo());
    }

    public Component getOptionsPanel() {
        JPanel checkBoxPanel = new JPanel(new GridLayout(2, 1));
        JCheckBox paintMargin = new JCheckBox("Paint Margin");
        paintMargin.setSelected(true);
        paintMargin.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _table.setPaintMarginBackground(e.getStateChange() == ItemEvent.SELECTED);
            }
        });
        checkBoxPanel.add(paintMargin);
        return checkBoxPanel;
    }

    public String getName() {
        return "PropertyPane Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_GRIDS;
    }

    public Component getDemoPanel() {
        JPanel panel = new JPanel(new BorderLayout());
        _table = createTable();
        PropertyPane propertyPane = new PropertyPane(_table);
        panel.add(propertyPane, BorderLayout.CENTER);
        return panel;
    }

    // create property table
    private PropertyTable createTable() {
        CellEditorManager.initDefaultEditor();
        // before use MaskFormatter, you must register converter context and editor context
        ConverterContext ssnConverterContext = null;
        try {
            ssnConverterContext = new ConverterContext("SSN", new MaskFormatter("###-##-####"));
        }
        catch (ParseException e) {
        }
        EditorContext ssnEditorContext = new EditorContext("SSN");
        CellEditorManager.registerEditor(String.class, new CellEditorFactory() {
            public CellEditor create() {
                return new FormattedTextFieldCellEditor(String.class);
            }
        }, ssnEditorContext);
        CellEditorManager.registerEditor(String.class, new CellEditorFactory() {
            public CellEditor create() {
                return new FileNameCellEditor() {
                    protected FileNameChooserComboBox createFileNameChooserComboBox() {
                        return new FileNameChooserComboBox() {
                            public PopupPanel createPopupComponent() {
                                FileChooserPanel panel = new FileChooserPanel() {
                                    protected JFileChooser createFileChooser() {
                                        JFileChooser fileChooser = new JFileChooser();
                                        fileChooser.setFileFilter(new FileFilter() {
                                            public boolean accept(File f) {
                                                return f.isDirectory() || f.getName().endsWith(".java");
                                            }

                                            public String getDescription() {
                                                return "Java files (*.java)";
                                            }
                                        });
                                        return fileChooser;
                                    }
                                };
                                panel.setTitle("Choose a file");
                                return panel;
                            }
                        };
                    }
                };
            }
        }, FileNameCellEditor.CONTEXT);

        CellEditorManager.registerEditor(String.class, new CellEditorFactory() {
            public CellEditor create() {
                return new FontNameCellEditor() {
                    protected ListComboBox createListComboBox(ComboBoxModel model, Class type) {
                        ListComboBox listComboBox = super.createListComboBox(model, type);
                        listComboBox.setEditable(false);
                        return listComboBox;
                    }
                };
            }
        }, new EditorContext("FontName-Noneditable"));

        final EnumConverter priorityConverter = new EnumConverter("Priority", Integer.class,
                new Object[]{
                        new Integer(0),
                        new Integer(1),
                        new Integer(2),
                        new Integer(3)
                },
                new String[]{
                        "Low",
                        "Normal",
                        "High",
                        "Urgent"},
                new Integer(1));
        ObjectConverterManager.registerConverter(priorityConverter.getType(), priorityConverter, priorityConverter.getContext());
        EnumCellRenderer priorityRenderer = new EnumCellRenderer(priorityConverter);
        CellRendererManager.registerRenderer(priorityConverter.getType(), priorityRenderer, priorityRenderer.getContext());
        CellEditorManager.registerEditor(priorityConverter.getType(), new CellEditorFactory() {
            public CellEditor create() {
                return new EnumCellEditor(priorityConverter);
            }
        }, new EditorContext(priorityConverter.getName()));

        CellEditorManager.registerEditor(String.class, new CellEditorFactory() {
            public CellEditor create() {
                return new MultilineStringCellEditor();
            }
        }, MultilineStringCellEditor.CONTEXT);
        TableCellRenderer multilineCellRenderer = new MultilineStringCellRenderer();
        CellRendererManager.registerRenderer(String.class, multilineCellRenderer, MultilineStringCellEditor.CONTEXT);

        ArrayList list = new ArrayList();

        SampleProperty property = null;

        property = new SampleProperty("Background", "The row is intended to show how to create a cell to input color in RGB format.", Color.class, "Appearance");
        list.add(property);

        property = new SampleProperty("Foreground", "The row is intended to show how to create a cell to input color in HEX format.", Color.class, "Appearance", ColorConverter.CONTEXT_HEX);
        list.add(property);

        property = new SampleProperty("Opaque", "The row is intended to show how to create a cell to input a boolean value. This one use combobox to choose a boolean type", Boolean.class, "Appearance");
        list.add(property);

        property = new SampleProperty("Bounds", "The row is intended to show how to create a cell to input a Rectangle.", Rectangle.class, "Appearance");
        list.add(property);

        property = new SampleProperty("Dimension", "The row is intended to show how to create a cell to input a Dimension.", Dimension.class, "Appearance");
        list.add(property);

        property = new SampleProperty("Name", "Name of the component", String.class);
        property.setCategory("Appearance");
        list.add(property);

        property = new SampleProperty("Font Name", "The row is intended to show how to create a cell to choose a font name", String.class);
        property.setEditorContext(new EditorContext("FontName"));
        property.setConverterContext(new ConverterContext("FontName"));
        property.setCategory("Appearance");
        list.add(property);

        property = new SampleProperty("Font", "The row is intended to show how to create a cell to choose a font", Font.class);
        property.setCategory("Appearance");
        list.add(property);

        property = new SampleProperty("Default Font Name", "The row is intended to show how to create a cell to choose a font name using a non-editable ListComoboBox", String.class);
        property.setEditorContext(new EditorContext("FontName-Noneditable"));
        property.setConverterContext(new ConverterContext("FontName"));
        property.setCategory("Appearance");
        list.add(property);

        property = new SampleProperty("Text", "Text of the component", String.class);
        property.setCategory("Appearance");
        list.add(property);

        property = new SampleProperty("Visible", "The row is intended to show how to create a cell to input a boolean value. This one use check box to choose a boolean type", Boolean.class);
        property.setEditorContext(BooleanCheckBoxCellEditor.CONTEXT);
        property.setCategory("Appearance");
        list.add(property);

        property = new SampleProperty("File Name", "The row is intended to show how to create a cell to input a file using FileChooser.", String.class);
        property.setEditorContext(FileNameCellEditor.CONTEXT);
        list.add(property);

        property = new SampleProperty("CreationDate", "The row is intended to show how to create a cell to input Date using DateComboBox.", Calendar.class);
        list.add(property);

        property = new SampleProperty("ExpirationDate", "The row is intended to show how to create a cell to input month/year using MonthComboBox.", Calendar.class);
        property.setConverterContext(MonthConverter.CONTEXT_MONTH);
        property.setEditorContext(MonthCellEditor.CONTEXT);
        list.add(property);

        property = new SampleProperty("DateTime", "The row is intended to show how to create a cell to input date and time using DateComboBox.", Calendar.class);
        property.setConverterContext(DateConverter.DATETIME_CONTEXT);
        property.setEditorContext(DateCellEditor.DATETIME_CONTEXT);
        list.add(property);

        property = new SampleProperty("Not Editable", "The row is intended to show a readonly cell.");
        property.setEditable(false);
        list.add(property);

        property = new SampleProperty("Double", "The row is intended to show how to create a cell to input a double.", Double.class);
        list.add(property);

        property = new SampleProperty("Float", "The row is intended to show how to create a cell to input a float.", Float.class);
        list.add(property);

        property = new SampleProperty("Integer", "The row is intended to show how to create a cell to input an integer.", Integer.class);
        list.add(property);

        property = new SampleProperty("Long", "The row is intended to show how to create a cell to input a long.", Long.class);
        list.add(property);

        property = new SampleProperty("Long text to test tooltips. If you see the whole text, you are OK.", "The row is intended to show how to tooltip of the cell value is too long.");
        list.add(property);

        property = new SampleProperty("SSN", "This row is intended to show how to create a String cell editor which uses MaskFormatter",
                String.class);
        property.setConverterContext(ssnConverterContext);
        property.setEditorContext(ssnEditorContext);
        list.add(property);

        property = new SampleProperty("Hidden", "This row is intended to hide a hidden property",
                String.class);
        property.setHidden(true);
        list.add(property);

        property = new SampleProperty("Level 1", "The row is intended to show how to create several levels.");

        SampleProperty property2 = new SampleProperty("Level 2");
        property.addChild(property2);

        SampleProperty property31 = new SampleProperty("Level 3.1");
        property2.addChild(property31);

        SampleProperty property32 = new SampleProperty("Level 3.2");
        property2.addChild(property32);

        SampleProperty property33 = new SampleProperty("Level 3.3 - Hidden");
        property33.setHidden(true);
        property2.addChild(property33);

        SampleProperty property34 = new SampleProperty("You can have as many levels as you want.");
        property2.addChild(property34);

        list.add(property);

        property = new SampleProperty("String Array", "This row shows you how to create a String array cell editor",
                String[].class);
        list.add(property);

        property = new SampleProperty("Priority", "This row shows you how to create a cell editor for enum",
                Integer.class);
        property.setConverterContext(priorityConverter.getContext());
        property.setEditorContext(new EditorContext(priorityConverter.getName()));
        list.add(property);

        property = new SampleProperty("Multiline", "This row shows you how to create a cell editor for multiple line text",
                String.class);
        property.setEditorContext(MultilineStringCellEditor.CONTEXT);
        list.add(property);

        property = new SampleProperty("Password", "This row shows you how to create a cell editor for password",
                char[].class);
        property.setConverterContext(PasswordConverter.CONTEXT);
        property.setEditorContext(PasswordCellEditor.CONTEXT);
        list.add(property);

        PropertyTableModel model = new StripePropertyTableModel(list);
        if (Lm.PG_DEBUG) {
            model.addPropertyChangeListener(new PropertyChangeListener() {
                public void propertyChange(PropertyChangeEvent evt) {
                    System.out.println("Model: " + evt.getPropertyName() + "--" + evt.getOldValue() + "=>" + evt.getNewValue());
                }
            });
        }
        PropertyTable table = new PropertyTable(model);
        table.setPreferredScrollableViewportSize(new Dimension(400, 500));
        table.expandFirstLevel();
// Support for tree expansion listener
//        table.addTreeExpansionListener(new TreeExpansionListener() {
//            public void treeCollapsed(TreeExpansionEvent event) {
//                System.out.println("-" + event.getPath());
//            }
//
//            public void treeExpanded(TreeExpansionEvent event) {
//                System.out.println("+" + event.getPath());
//            }
//        });
        PropertyTableSearchable searchable = new PropertyTableSearchable(table);
        searchable.setRecursive(true);
        return table;
    }

    class StripePropertyTableModel extends PropertyTableModel implements StyleModel {
        public StripePropertyTableModel() {
        }

        public StripePropertyTableModel(List properties) {
            super(properties);
        }

        public StripePropertyTableModel(List properties, String[] categories) {
            super(properties, categories);
        }

        public StripePropertyTableModel(List properties, int categoryOrder) {
            super(properties, categoryOrder);
        }

        protected final Color BACKGROUND1 = new Color(253, 253, 244);
        protected final Color BACKGROUND2 = new Color(255, 255, 255);

        CellStyle cellStyle = new CellStyle();

        public CellStyle getCellStyleAt(int rowIndex, int columnIndex) {
            cellStyle.setHorizontalAlignment(-1);
            cellStyle.setForeground(Color.BLACK);
            if (rowIndex % 2 == 0) {
                cellStyle.setBackground(BACKGROUND1);
            }
            else {
                cellStyle.setBackground(BACKGROUND2);
            }
            return cellStyle;
        }

        public boolean isCellStyleOn() {
            return true;
        }
    }

    static HashMap map = new HashMap();

    static {
        map.put("Bounds", new Rectangle(0, 0, 100, 200));
        map.put("Dimension", new Dimension(800, 600));
        map.put("Background", new Color(255, 0, 0));
        map.put("Foreground", new Color(255, 255, 255));
        map.put("File Name", "C:\\Program Files\\JIDE\\src\\com\\jidesoft\\Demo.java");
        map.put("CreationDate", Calendar.getInstance());
        map.put("ExpirationDate", Calendar.getInstance());
        map.put("DateTime", Calendar.getInstance());
        map.put("Name", "Label1");
        map.put("Font Name", "Arial");
        map.put("Font", new Font("Tahoma", Font.BOLD, 11));
        map.put("Default Font Name", "Verdana");
        map.put("Text", "Data");
        map.put("Opaque", Boolean.FALSE);
        map.put("Visible", Boolean.TRUE);
        map.put("Not Editable", new Integer(10));
        map.put("Long", new Long(123456789));
        map.put("Integer", new Integer(1234));
        map.put("Double", new Double(1.0));
        map.put("Float", new Float(0.01));
        map.put("SSN", "000-00-0000");
        map.put("Priority", new Integer(1));
        map.put("Multiline", "This is a multiple line cell editor. \nA new line starts here.");
        map.put("Password", new char[]{'p', 'a', 's', 's', 'w', 'o', 'r', 'd'});
    }

    static class SampleProperty extends Property {
        public SampleProperty(String name, String description, Class type, String category, ConverterContext context, java.util.List childProperties) {
            super(name, description, type, category, context, childProperties);
            init();
        }

        public SampleProperty(String name, String description, Class type, String category, ConverterContext context) {
            super(name, description, type, category, context);
            init();
        }

        public SampleProperty(String name, String description, Class type, String category) {
            super(name, description, type, category);
            init();
        }

        public SampleProperty(String name, String description, Class type) {
            super(name, description, type);
            init();
        }

        public SampleProperty(String name, String description) {
            super(name, description);
            init();
        }

        public SampleProperty(String name) {
            super(name);
            init();
        }

        private void init() {
            if (Lm.PG_DEBUG) {
                addPropertyChangeListener(new PropertyChangeListener() {
                    public void propertyChange(PropertyChangeEvent evt) {
                        System.out.println("Property: " + evt.getPropertyName() + "--" + evt.getOldValue() + "=>" + evt.getNewValue());
                    }
                });
            }
        }

        public void setValue(Object value) {
            if (Lm.PG_DEBUG) {
                System.out.println("(BeanProperty) Set value " + ObjectConverterManager.toString(value) + " of type " + (value != null ? value.getClass().getName() : "null"));
            }
            map.put(getFullName(), value);
        }

        public Object getValue() {
            Object value = map.get(getFullName());
            if (Lm.PG_DEBUG) {
                System.out.println("(BeanProperty) Get value " + ObjectConverterManager.toString(value));
            }
            return value;
        }

        public boolean hasValue() {
            return map.get(getFullName()) != null;
        }
    }
}
