/*
 * @(#)ResizingTableDemo.java 9/8/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.grid.JideTable;
import com.jidesoft.grid.SortableTable;
import com.jidesoft.grid.TableColumnChooser;
import com.jidesoft.grid.TableUtils;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JideSwingUtilities;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

/**
 * Demoed Component: {@link com.jidesoft.grid.JideTable}
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: any L&F
 */
public class ResizableTableDemo extends AbstractDemo {
    public JideTable _table;

    public ResizableTableDemo() {
    }

    public String getName() {
        return "Resizable Table Demo";
    }

    public String getProduct() {
        return PRODUCT_NAME_GRIDS;
    }

    public String getDescription() {
        return "This is a demo of JideTable with several resizable options.\n" +
                "\nWhen a table is column resizable or row resizable, you drag the grid line " +
                "in the table to resize the column or row respectively." +
                "If a table is column auto resizable, you can double click on the grid line of the table header " +
                "and make the table column automatically resize to the widest size of all cells in that column.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.grid.TableColumnAutoResizer\n" +
                "com.jidesoft.grid.TableRowAutoResizer\n" +
                "com.jidesoft.grid.TableColumnResizer\n" +
                "com.jidesoft.grid.TableRowResizer\n" +
                "com.jidesoft.grid.JideTable";
    }

    public Component getOptionsPanel() {
        JPanel switchPanel = new JPanel(new GridLayout(0, 1, 3, 3));

        switchPanel.add(new JLabel("Options: "));

        final JCheckBox option1 = new JCheckBox("Column Resizable");
        final JCheckBox option2 = new JCheckBox("Row Resizable");
        final JCheckBox option3 = new JCheckBox("Column Auto-resizable");
        final JCheckBox option4 = new JCheckBox("Enable Column Chooser");

        switchPanel.add(option1);
        switchPanel.add(option2);
        switchPanel.add(option3);
        switchPanel.add(option4);

        option1.setSelected(_table.isColumnResizable());
        option1.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _table.setColumnResizable(option1.isSelected());
            }
        });

        option2.setSelected(_table.isRowResizable());
        option2.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _table.setRowResizable(option2.isSelected());
            }
        });

        option3.setSelected(_table.isColumnAutoResizable());
        option3.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                _table.setColumnAutoResizable(option3.isSelected());
            }
        });

        option4.setSelected(true);
        TableColumnChooser.install(_table);
        option4.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                if (option4.isSelected()) {
                    TableColumnChooser.install(_table);
                }
                else {
                    TableColumnChooser.uninstall(_table);
                }
            }
        });

        switchPanel.add(JideSwingUtilities.createLeftPanel(new JButton(new AbstractAction("Auto resize all columns") {
            public void actionPerformed(ActionEvent e) {
                TableUtils.autoResizeAllColumns(_table);
            }
        })));
        return switchPanel;
    }

    static String[] QUOTE_COLUMNS = new String[]{"Symbol", "Name", "Last", "Change", "Volume"};

    static Object[][] QUOTES = new Object[][]{
            new Object[]{"AA", "ALCOA INC", "32.88", "+0.53 (1.64%)", "4156200"},
            new Object[]{"AIG", "AMER INTL GROUP", "69.53", "-0.58 (0.83%)", "4369200"},
            new Object[]{"AXP", "AMER EXPRESS CO", "48.90", "-0.35 (0.71%)", "4103600"},
            new Object[]{"BA", "BOEING CO", "49.14", "-0.18 (0.36%)", "3573700"},
            new Object[]{"C", "CITIGROUP", "44.21", "-0.89 (1.97%)", "28594900"},
            new Object[]{"CAT", "CATERPILLAR INC", "79.40", "+0.62 (0.79%)", "1458200"},
            new Object[]{"DD", "DU PONT CO", "42.62", "-0.14 (0.33%)", "1832700"},
            new Object[]{"DIS", "WALT DISNEY CO", "23.87", "-0.32 (1.32%)", "4443600"},
            new Object[]{"GE", "GENERAL ELEC CO", "33.37", "+0.24 (0.72%)", "31429500"},
            new Object[]{"GM", "GENERAL MOTORS", "43.94", "-0.20 (0.45%)", "3722100"},
            new Object[]{"HD", "HOME DEPOT INC", "34.33", "-0.18 (0.52%)", "5367900"},
            new Object[]{"HON", "HONEYWELL INTL", "35.70", "+0.23 (0.65%)", "4092100"},
            new Object[]{"HPQ", "HEWLETT-PACKARD", "19.65", "-0.25 (1.26%)", "11003000"},
            new Object[]{"IBM", "INTL BUS MACHINE", "84.02", "-0.11 (0.13%)", "6880500"},
            new Object[]{"INTC", "INTEL CORP", "23.15", "-0.23 (0.98%)", "95177008"},
            new Object[]{"JNJ", "JOHNSON&JOHNSON", "55.35", "-0.57 (1.02%)", "5428000"},
            new Object[]{"JPM", "JP MORGAN CHASE", "36.00", "-0.45 (1.23%)", "12135300"},
            new Object[]{"KO", "COCA COLA CO", "50.84", "-0.32 (0.63%)", "4143600"},
            new Object[]{"MCD", "MCDONALDS CORP", "27.91", "+0.12 (0.43%)", "6110800"},
            new Object[]{"MMM", "3M COMPANY", "88.62", "+0.43 (0.49%)", "2073800"},
            new Object[]{"MO", "ALTRIA GROUP", "48.20", "-0.80 (1.63%)", "6005500"},
            new Object[]{"MRK", "MERCK & CO", "44.71", "-0.97 (2.12%)", "5472100"},
            new Object[]{"MSFT", "MICROSOFT CP", "27.87", "-0.26 (0.92%)", "46717716"},
            new Object[]{"PFE", "PFIZER INC", "32.58", "-1.43 (4.20%)", "28783200"},
            new Object[]{"PG", "PROCTER & GAMBLE", "55.01", "-0.07 (0.13%)", "5538400"},
            new Object[]{"SBC", "SBC COMMS", "23.00", "-0.54 (2.29%)", "6423400"},
            new Object[]{"UTX", "UNITED TECH CP", "91.00", "+1.16 (1.29%)", "1868600"},
            new Object[]{"VZ", "VERIZON COMMS", "34.81", "-0.35 (1.00%)", "4182600"},
            new Object[]{"WMT", "WAL-MART STORES", "52.33", "-0.25 (0.48%)", "6776700"},
            new Object[]{"XOM", "EXXON MOBIL", "45.32", "-0.14 (0.31%)", "7838100"}
    };

    static class QuoteTableModel extends DefaultTableModel {
        public QuoteTableModel() {
            super(QUOTES, QUOTE_COLUMNS);
        }

        public boolean isCellEditable(int row, int column) {
            return true;
        }
    }

    public Component getDemoPanel() {
        JPanel panel = new JPanel(new BorderLayout(6, 6));
        _table = new SortableTable(new QuoteTableModel());
        _table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);

//        _table.putClientProperty(TableColumnChooser.SHOW_AUTO_RESIZE, Boolean.FALSE);
//        _table.putClientProperty(TableColumnChooser.FIXED_COLUMNS, new int[]{2,3});
        _table.putClientProperty(TableColumnChooser.DISPLAY_COLUMN_NAMES, new String[]{"Stock Symbol", "Company Name", "Last Stock Price", "Change (%)", "Volume"});

// uncomment the following line if you want auto resize do not consider the header column preferred width
//        _table.putClientProperty(TableUtils.AUTO_RESIZE_CONSIDER_HEADER, Boolean.FALSE);

        _table.setColumnResizable(true);
        _table.setRowResizable(true);

        _table.setColumnAutoResizable(true);

        _table.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
        _table.getColumnModel().getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);

        panel.add(new JScrollPane(_table), BorderLayout.CENTER);
        return panel;
    }

    public String getDemoFolder() {
        return "G12. JideTable";
    }

    public static void main(String[] args) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new ResizableTableDemo());
    }
}

