/*
 * @(#)TreeTableScrollPaneDemo.java 1/8/2006
 *
 * Copyright 2002 - 2006 JIDE Software Inc. All rights reserved.
 */

import com.jidesoft.grid.*;
import com.jidesoft.plaf.LookAndFeelFactory;
import com.jidesoft.swing.JideTabbedPane;

import javax.swing.*;
import javax.swing.table.TableModel;
import java.awt.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Demoed Component: {@link com.jidesoft.grid.TableScrollPane}
 * <br>
 * Required jar files: jide-common.jar, jide-grids.jar
 * <br>
 * Required L&F: any L&F
 */
public class TreeTableScrollPaneDemo extends AbstractDemo {

    protected MultiTableModel _model;

    public TreeTableScrollPaneDemo() {
    }

    public String getName() {
        return "TableScrollPane (TreeTable)";
    }

    public String getDescription() {
        return "This is a demo of TableScrollPane. TableScrollPane is a special component which supports table row header, row footer and column footer.\n" +
                "\nIn this particular demo, we show you how to use TreeTable in row header in a TableScrollPane.\n" +
                "\n" +
                "Demoed classes:\n" +
                "com.jidesoft.grid.TableScrollPane\n" +
                "com.jidesoft.grid.JideTable";
    }

    public String getProduct() {
        return PRODUCT_NAME_GRIDS;
    }

    public int getAttributes() {
        return ATTRIBUTE_NEW;
    }

    public Component getDemoPanel() {
        JPanel panel = new JPanel();
        panel.setPreferredSize(new Dimension(800, 400));
        panel.setLayout(new BorderLayout());
        panel.add(createTabbedPane(), BorderLayout.CENTER);
        return panel;
    }

    private Component createTabbedPane() {
        JideTabbedPane tabbedPane = new JideTabbedPane();
        tabbedPane.setBoxStyleTab(true);
        tabbedPane.addTab("TreeTable in TableScrollPane", createTablePane());
        return tabbedPane;

    }

    private TableScrollPane createTablePane() {
        List list = new ArrayList();
        DummyRow w1 = new DummyRow("Week 1");
        for (int i = 0; i < 7; i++) {
            w1.addChild(new DummyRow("Day " + (i + 1)));
        }
        list.add(w1);
        DummyRow w2 = new DummyRow("Week 2");
        for (int i = 0; i < 7; i++) {
            w2.addChild(new DummyRow("Day " + (i + 1)));
        }
        list.add(w2);
        DummyRow w3 = new DummyRow("Week 3");
        for (int i = 0; i < 7; i++) {
            w3.addChild(new DummyRow("Day " + (i + 1)));
        }
        list.add(w3);

        _model = new TreeTableModel(list) {
            public int getColumnType(int columnIndex) {
                return columnIndex < 1 ? HEADER_COLUMN : REGULAR_COLUMN;
            }

            public String getColumnName(int column) {
                if (column == 0) {
                    return "Day";
                }
                else {
                    return "Product " + column;
                }
            }

            public int getColumnCount() {
                return 21;
            }
        };
        TableScrollPane pane = new TableScrollPane(_model, true) {
            protected JTable createTable(final TableModel model, final boolean sortable, final int type, final int tableIndex) {
                if (type == MultiTableModel.HEADER_COLUMN) {
                    TreeTable treeTable = new TreeTable(model);
                    treeTable.setRowHeight(18);
                    treeTable.setShowTreeLines(true);
                    return treeTable;
                }
                else {
                    JTable table = super.createTable(model, sortable, type, tableIndex);
                    table.setRowHeight(18);
                    return table;
                }
            }
        };
        return pane;
    }

    public String getDemoFolder() {
        return "G10. TableScrollPane";
    }

    static public void main(String[] s) {
        LookAndFeelFactory.installDefaultLookAndFeelAndExtension();
        showAsFrame(new TreeTableScrollPaneDemo());
    }

    static class DummyRow extends DefaultExpandableRow {

        private String _name;
        private List _values;

        public DummyRow(String name) {
            _name = name;
            _values = new ArrayList();
            for (int i = 0; i < 20; i ++) {
                _values.add(new Integer((int) (Math.random() * 40)));
            }
        }

        public Object getValueAt(int columnIndex) {
            if (columnIndex == 0) {
                return _name;
            }
            return _values.get(columnIndex - 1);
        }
    }
}
